@@ -20,6 +20,10 @@
  * @file
  */
 
+use MediaWiki\MainConfigNames;
+use Wikimedia\ParamValidator\ParamValidator;
+use Wikimedia\ParamValidator\TypeDef\IntegerDef;
+
 /**
  * This action allows users to get their watchlist items in RSS/Atom formats.
  * When executed, it performs a nested call to the API to get the needed data,
@@ -32,6 +36,23 @@ class ApiFeedWatchlist extends ApiBase {
 	private $watchlistModule = null;
 	private $linkToSections = false;
 
+	/** @var Parser */
+	private $parser;
+
+	/**
+	 * @param ApiMain $main
+	 * @param string $action
+	 * @param Parser $parser
+	 */
+	public function __construct(
+		ApiMain $main,
+		$action,
+		Parser $parser
+	) {
+		parent::__construct( $main, $action );
+		$this->parser = $parser;
+	}
+
 	/**
 	 * This module uses a custom feed wrapper printer.
 	 *
@@ -47,12 +68,13 @@ class ApiFeedWatchlist extends ApiBase {
 	 */
 	public function execute() {
 		$config = $this->getConfig();
-		$feedClasses = $config->get( 'FeedClasses' );
+		$feedClasses = $config->get( MainConfigNames::FeedClasses );
 		$params = [];
+		$feedItems = [];
 		try {
 			$params = $this->extractRequestParams();
 
-			if ( !$config->get( 'Feed' ) ) {
+			if ( !$config->get( MainConfigNames::Feed ) ) {
 				$this->dieWithError( 'feed-unavailable' );
 			}
 
@@ -61,7 +83,7 @@ class ApiFeedWatchlist extends ApiBase {
 			}
 
 			// limit to the number of hours going from now back
-			$endTime = wfTimestamp( TS_MW, time() - intval( $params['hours'] * 60 * 60 ) );
+			$endTime = wfTimestamp( TS_MW, time() - (int)$params['hours'] * 60 * 60 );
 
 			// Prepare parameters for nested request
 			$fauxReqArr = [
@@ -69,10 +91,10 @@ class ApiFeedWatchlist extends ApiBase {
 				'meta' => 'siteinfo',
 				'siprop' => 'general',
 				'list' => 'watchlist',
-				'wlprop' => 'title|user|comment|timestamp|ids',
+				'wlprop' => 'title|user|comment|timestamp|ids|loginfo',
 				'wldir' => 'older', // reverse order - from newest to oldest
 				'wlend' => $endTime, // stop at this time
-				'wllimit' => min( 50, $this->getConfig()->get( 'FeedLimit' ) )
+				'wllimit' => min( 50, $this->getConfig()->get( MainConfigNames::FeedLimit ) )
 			];
 
 			if ( $params['wlowner'] !== null ) {
@@ -102,15 +124,12 @@ class ApiFeedWatchlist extends ApiBase {
 				$fauxReqArr['wlallrev'] = '';
 			}
 
-			// Create the request
 			$fauxReq = new FauxRequest( $fauxReqArr );
 
-			// Execute
 			$module = new ApiMain( $fauxReq );
 			$module->execute();
 
 			$data = $module->getResult()->getResultData( [ 'query', 'watchlist' ] );
-			$feedItems = [];
 			foreach ( (array)$data as $key => $info ) {
 				if ( ApiResult::isMetadataKey( $key ) ) {
 					continue;
@@ -121,10 +140,10 @@ class ApiFeedWatchlist extends ApiBase {
 				}
 			}
 
-			$msg = wfMessage( 'watchlist' )->inContentLanguage()->text();
+			$msg = $this->msg( 'watchlist' )->inContentLanguage()->text();
 
-			$feedTitle = $this->getConfig()->get( 'Sitename' ) . ' - ' . $msg .
-				' [' . $this->getConfig()->get( 'LanguageCode' ) . ']';
+			$feedTitle = $this->getConfig()->get( MainConfigNames::Sitename ) . ' - ' . $msg .
+				' [' . $this->getConfig()->get( MainConfigNames::LanguageCode ) . ']';
 			$feedUrl = SpecialPage::getTitleFor( 'Watchlist' )->getFullURL();
 
 			$feed = new $feedClasses[$params['feedformat']] (
@@ -139,17 +158,18 @@ class ApiFeedWatchlist extends ApiBase {
 			$this->getMain()->setCacheMaxAge( 0 );
 
 			// @todo FIXME: Localise  brackets
-			$feedTitle = $this->getConfig()->get( 'Sitename' ) . ' - Error - ' .
-				wfMessage( 'watchlist' )->inContentLanguage()->text() .
-				' [' . $this->getConfig()->get( 'LanguageCode' ) . ']';
+			$feedTitle = $this->getConfig()->get( MainConfigNames::Sitename ) . ' - Error - ' .
+				$this->msg( 'watchlist' )->inContentLanguage()->text() .
+				' [' . $this->getConfig()->get( MainConfigNames::LanguageCode ) . ']';
 			$feedUrl = SpecialPage::getTitleFor( 'Watchlist' )->getFullURL();
 
-			$feedFormat = isset( $params['feedformat'] ) ? $params['feedformat'] : 'rss';
-			$msg = wfMessage( 'watchlist' )->inContentLanguage()->escaped();
+			$feedFormat = $params['feedformat'] ?? 'rss';
+			$msg = $this->msg( 'watchlist' )->inContentLanguage()->escaped();
 			$feed = new $feedClasses[$feedFormat] ( $feedTitle, $msg, $feedUrl );
 
 			if ( $e instanceof ApiUsageException ) {
 				foreach ( $e->getStatusValue()->getErrors() as $error ) {
+					// @phan-suppress-next-line PhanUndeclaredMethod
 					$msg = ApiMessage::create( $error )
 						->inLanguage( $this->getLanguage() );
 					$errorTitle = $this->msg( 'api-feed-error-title', $msg->getApiCode() );
@@ -157,12 +177,8 @@ class ApiFeedWatchlist extends ApiBase {
 					$feedItems[] = new FeedItem( $errorTitle, $errorText, '', '', '' );
 				}
 			} else {
-				if ( $e instanceof UsageException ) {
-					$errorCode = $e->getCodeString();
-				} else {
-					// Something is seriously wrong
-					$errorCode = 'internal_api_error';
-				}
+				// Something is seriously wrong
+				$errorCode = 'internal_api_error';
 				$errorTitle = $this->msg( 'api-feed-error-title', $errorCode );
 				$errorText = $e->getMessage();
 				$feedItems[] = new FeedItem( $errorTitle, $errorText, '', '', '' );
@@ -174,7 +190,7 @@ class ApiFeedWatchlist extends ApiBase {
 
 	/**
 	 * @param array $info
-	 * @return FeedItem
+	 * @return FeedItem|null
 	 */
 	private function createFeedItem( $info ) {
 		if ( !isset( $info['title'] ) ) {
@@ -197,11 +213,16 @@ class ApiFeedWatchlist extends ApiBase {
 			}
 		}
 		if ( isset( $info['revid'] ) ) {
-			$titleUrl = $title->getFullURL( [ 'diff' => $info['revid'] ] );
+			if ( $info['revid'] === 0 && isset( $info['logid'] ) ) {
+				$logTitle = Title::makeTitle( NS_SPECIAL, 'Log' );
+				$titleUrl = $logTitle->getFullURL( [ 'logid' => $info['logid'] ] );
+			} else {
+				$titleUrl = $title->getFullURL( [ 'diff' => $info['revid'] ] );
+			}
 		} else {
 			$titleUrl = $title->getFullURL( $curidParam );
 		}
-		$comment = isset( $info['comment'] ) ? $info['comment'] : null;
+		$comment = $info['comment'] ?? null;
 
 		// Create an anchor to section.
 		// The anchor won't work for sections that have dupes on page
@@ -210,8 +231,7 @@ class ApiFeedWatchlist extends ApiBase {
 		if ( $this->linkToSections && $comment !== null &&
 			preg_match( '!(.*)/\*\s*(.*?)\s*\*/(.*)!', $comment, $matches )
 		) {
-			global $wgParser;
-			$titleUrl .= $wgParser->guessSectionNameFromWikiText( $matches[ 2 ] );
+			$titleUrl .= $this->parser->guessSectionNameFromWikiText( $matches[ 2 ] );
 		}
 
 		$timestamp = $info['timestamp'];
@@ -237,17 +257,17 @@ class ApiFeedWatchlist extends ApiBase {
 	}
 
 	public function getAllowedParams( $flags = 0 ) {
-		$feedFormatNames = array_keys( $this->getConfig()->get( 'FeedClasses' ) );
+		$feedFormatNames = array_keys( $this->getConfig()->get( MainConfigNames::FeedClasses ) );
 		$ret = [
 			'feedformat' => [
-				ApiBase::PARAM_DFLT => 'rss',
-				ApiBase::PARAM_TYPE => $feedFormatNames
+				ParamValidator::PARAM_DEFAULT => 'rss',
+				ParamValidator::PARAM_TYPE => $feedFormatNames
 			],
 			'hours' => [
-				ApiBase::PARAM_DFLT => 24,
-				ApiBase::PARAM_TYPE => 'integer',
-				ApiBase::PARAM_MIN => 1,
-				ApiBase::PARAM_MAX => 72,
+				ParamValidator::PARAM_DEFAULT => 24,
+				ParamValidator::PARAM_TYPE => 'integer',
+				IntegerDef::PARAM_MIN => 1,
+				IntegerDef::PARAM_MAX => 72,
 			],
 			'linktosections' => false,
 		];
@@ -261,19 +281,20 @@ class ApiFeedWatchlist extends ApiBase {
 			'excludeuser' => 'wlexcludeuser',
 		];
 		if ( $flags ) {
+			// @phan-suppress-next-line PhanParamTooMany
 			$wlparams = $this->getWatchlistModule()->getAllowedParams( $flags );
 			foreach ( $copyParams as $from => $to ) {
 				$p = $wlparams[$from];
 				if ( !is_array( $p ) ) {
-					$p = [ ApiBase::PARAM_DFLT => $p ];
+					$p = [ ParamValidator::PARAM_DEFAULT => $p ];
 				}
 				if ( !isset( $p[ApiBase::PARAM_HELP_MSG] ) ) {
 					$p[ApiBase::PARAM_HELP_MSG] = "apihelp-query+watchlist-param-$from";
 				}
-				if ( isset( $p[ApiBase::PARAM_TYPE] ) && is_array( $p[ApiBase::PARAM_TYPE] ) &&
+				if ( isset( $p[ParamValidator::PARAM_TYPE] ) && is_array( $p[ParamValidator::PARAM_TYPE] ) &&
 					isset( $p[ApiBase::PARAM_HELP_MSG_PER_VALUE] )
 				) {
-					foreach ( $p[ApiBase::PARAM_TYPE] as $v ) {
+					foreach ( $p[ParamValidator::PARAM_TYPE] as $v ) {
 						if ( !isset( $p[ApiBase::PARAM_HELP_MSG_PER_VALUE][$v] ) ) {
 							$p[ApiBase::PARAM_HELP_MSG_PER_VALUE][$v] = "apihelp-query+watchlist-paramvalue-$from-$v";
 						}
@@ -282,7 +303,7 @@ class ApiFeedWatchlist extends ApiBase {
 				$ret[$to] = $p;
 			}
 		} else {
-			foreach ( $copyParams as $from => $to ) {
+			foreach ( $copyParams as $to ) {
 				$ret[$to] = null;
 			}
 		}
