@@ -23,6 +23,11 @@
  * @since 1.24
  */
 
+use MediaWiki\Linker\LinksMigration;
+use MediaWiki\MainConfigNames;
+use Wikimedia\ParamValidator\ParamValidator;
+use Wikimedia\ParamValidator\TypeDef\IntegerDef;
+
 /**
  * This implements prop=redirects, prop=linkshere, prop=catmembers,
  * prop=transcludedin, and prop=fileusage
@@ -32,7 +37,7 @@
  */
 class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 
-	// Data for the various modules implemented by this class
+	/** @var array Data for the various modules implemented by this class */
 	private static $settings = [
 		'redirects' => [
 			'code' => 'rd',
@@ -59,7 +64,6 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 			'code' => 'ti',
 			'prefix' => 'tl',
 			'linktable' => 'templatelinks',
-			'indexes' => [ 'tl_namespace', 'tl_backlinks_namespace' ],
 			'from_namespace' => true,
 			'showredirects' => true,
 		],
@@ -75,8 +79,21 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 		],
 	];
 
-	public function __construct( ApiQuery $query, $moduleName ) {
+	/** @var LinksMigration */
+	private $linksMigration;
+
+	/**
+	 * @param ApiQuery $query
+	 * @param string $moduleName
+	 * @param LinksMigration $linksMigration
+	 */
+	public function __construct(
+		ApiQuery $query,
+		$moduleName,
+		LinksMigration $linksMigration
+	) {
 		parent::__construct( $query, $moduleName, self::$settings[$moduleName]['code'] );
+		$this->linksMigration = $linksMigration;
 	}
 
 	public function execute() {
@@ -95,16 +112,16 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 
 		$db = $this->getDB();
 		$params = $this->extractRequestParams();
-		$prop = array_flip( $params['prop'] );
+		$prop = array_fill_keys( $params['prop'], true );
 		$emptyString = $db->addQuotes( '' );
 
 		$pageSet = $this->getPageSet();
-		$titles = $pageSet->getGoodAndMissingTitles();
+		$titles = $pageSet->getGoodAndMissingPages();
 		$map = $pageSet->getGoodAndMissingTitlesByNamespace();
 
 		// Add in special pages, they can theoretically have backlinks too.
 		// (although currently they only do for prop=redirects)
-		foreach ( $pageSet->getSpecialTitles() as $id => $title ) {
+		foreach ( $pageSet->getSpecialPages() as $id => $title ) {
 			$titles[] = $title;
 			$map[$title->getNamespace()][$title->getDBkey()] = $id;
 		}
@@ -113,13 +130,18 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 		$p = $settings['prefix'];
 		$hasNS = !isset( $settings['to_namespace'] );
 		if ( $hasNS ) {
-			$bl_namespace = "{$p}_namespace";
-			$bl_title = "{$p}_title";
+			if ( isset( $this->linksMigration::$mapping[$settings['linktable']] ) ) {
+				list( $bl_namespace, $bl_title ) = $this->linksMigration->getTitleFields( $settings['linktable'] );
+			} else {
+				$bl_namespace = "{$p}_namespace";
+				$bl_title = "{$p}_title";
+			}
 		} else {
+			// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 			$bl_namespace = $settings['to_namespace'];
 			$bl_title = "{$p}_to";
 
-			$titles = array_filter( $titles, function ( $t ) use ( $bl_namespace ) {
+			$titles = array_filter( $titles, static function ( $t ) use ( $bl_namespace ) {
 				return $t->getNamespace() === $bl_namespace;
 			} );
 			$map = array_intersect_key( $map, [ $bl_namespace => true ] );
@@ -155,7 +177,7 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 		$miser_ns = null;
 		if ( $params['namespace'] !== null ) {
 			if ( empty( $settings['from_namespace'] ) ) {
-				if ( $this->getConfig()->get( 'MiserMode' ) ) {
+				if ( $this->getConfig()->get( MainConfigNames::MiserMode ) ) {
 					$miser_ns = $params['namespace'];
 				} else {
 					$this->addWhereFld( 'page_namespace', $params['namespace'] );
@@ -202,7 +224,21 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 		}
 
 		// Populate the rest of the query
-		$this->addTables( [ $settings['linktable'], 'page' ] );
+		list( $idxNoFromNS, $idxWithFromNS ) = $settings['indexes'] ?? [ '', '' ];
+		// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
+		if ( isset( $this->linksMigration::$mapping[$settings['linktable']] ) ) {
+			// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
+			$queryInfo = $this->linksMigration->getQueryInfo( $settings['linktable'] );
+			$this->addTables( array_merge( [ 'page' ], $queryInfo['tables'] ) );
+			$this->addJoinConds( $queryInfo['joins'] );
+			// TODO: Move to links migration
+			if ( in_array( 'linktarget', $queryInfo['tables'] ) ) {
+				$idxWithFromNS .= '_target_id';
+			}
+		} else {
+			// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
+			$this->addTables( [ $settings['linktable'], 'page' ] );
+		}
 		$this->addWhere( "$bl_from = page_id" );
 
 		if ( $this->getModuleName() === 'redirects' ) {
@@ -245,7 +281,7 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 
 		if ( $params['show'] !== null ) {
 			// prop=redirects only
-			$show = array_flip( $params['show'] );
+			$show = array_fill_keys( $params['show'], true );
 			if ( isset( $show['fragment'] ) && isset( $show['!fragment'] ) ||
 				isset( $show['redirect'] ) && isset( $show['!redirect'] )
 			) {
@@ -267,25 +303,22 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 		// (...)" and chooses the wrong index, so specify the correct index to
 		// use for the query. See T139056 for details.
 		if ( !empty( $settings['indexes'] ) ) {
-			list( $idxNoFromNS, $idxWithFromNS ) = $settings['indexes'];
 			if ( $params['namespace'] !== null && !empty( $settings['from_namespace'] ) ) {
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 				$this->addOption( 'USE INDEX', [ $settings['linktable'] => $idxWithFromNS ] );
-			} else {
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
+			} elseif ( !isset( $this->linksMigration::$mapping[$settings['linktable']] ) ) {
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 				$this->addOption( 'USE INDEX', [ $settings['linktable'] => $idxNoFromNS ] );
 			}
 		}
 
-		// MySQL (or at least 5.5.5-10.0.23-MariaDB) chooses a really bad query
-		// plan if it thinks there will be more matching rows in the linktable
-		// than are in page. Use STRAIGHT_JOIN here to force it to use the
-		// intended, fast plan. See T145079 for details.
-		$this->addOption( 'STRAIGHT_JOIN' );
-
 		$this->addOption( 'LIMIT', $params['limit'] + 1 );
 
 		$res = $this->select( __METHOD__ );
 
 		if ( $resultPageSet === null ) {
+			// @phan-suppress-next-line PhanPossiblyUndeclaredVariable set when used
 			if ( $fld_title ) {
 				$this->executeGenderCacheFromResultWrapper( $res, __METHOD__ );
 			}
@@ -308,6 +341,7 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 				$id = $map[$row->bl_namespace][$row->bl_title];
 
 				$vals = [];
+				// @phan-suppress-next-line PhanPossiblyUndeclaredVariable set when used
 				if ( $fld_pageid ) {
 					$vals['pageid'] = (int)$row->page_id;
 				}
@@ -316,9 +350,11 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 						Title::makeTitle( $row->page_namespace, $row->page_title )
 					);
 				}
+				// @phan-suppress-next-line PhanPossiblyUndeclaredVariable set when used
 				if ( $fld_fragment && $row->rd_fragment !== null && $row->rd_fragment !== '' ) {
 					$vals['fragment'] = $row->rd_fragment;
 				}
+				// @phan-suppress-next-line PhanPossiblyUndeclaredVariable set when used
 				if ( $fld_redirect ) {
 					$vals['redirect'] = (bool)$row->page_is_redirect;
 				}
@@ -367,44 +403,45 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 
 		$ret = [
 			'prop' => [
-				ApiBase::PARAM_TYPE => [
+				ParamValidator::PARAM_TYPE => [
 					'pageid',
 					'title',
 				],
-				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_DFLT => 'pageid|title',
+				ParamValidator::PARAM_ISMULTI => true,
+				ParamValidator::PARAM_DEFAULT => 'pageid|title',
 				ApiBase::PARAM_HELP_MSG_PER_VALUE => [],
 			],
 			'namespace' => [
-				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_TYPE => 'namespace',
+				ParamValidator::PARAM_ISMULTI => true,
+				ParamValidator::PARAM_TYPE => 'namespace',
 			],
 			'show' => null, // Will be filled/removed below
 			'limit' => [
-				ApiBase::PARAM_DFLT => 10,
-				ApiBase::PARAM_TYPE => 'limit',
-				ApiBase::PARAM_MIN => 1,
-				ApiBase::PARAM_MAX => ApiBase::LIMIT_BIG1,
-				ApiBase::PARAM_MAX2 => ApiBase::LIMIT_BIG2
+				ParamValidator::PARAM_DEFAULT => 10,
+				ParamValidator::PARAM_TYPE => 'limit',
+				IntegerDef::PARAM_MIN => 1,
+				IntegerDef::PARAM_MAX => ApiBase::LIMIT_BIG1,
+				IntegerDef::PARAM_MAX2 => ApiBase::LIMIT_BIG2
 			],
 			'continue' => [
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-continue',
 			],
 		];
 
-		if ( empty( $settings['from_namespace'] ) && $this->getConfig()->get( 'MiserMode' ) ) {
+		if ( empty( $settings['from_namespace'] ) &&
+		$this->getConfig()->get( MainConfigNames::MiserMode ) ) {
 			$ret['namespace'][ApiBase::PARAM_HELP_MSG_APPEND] = [
 				'api-help-param-limited-in-miser-mode',
 			];
 		}
 
 		if ( !empty( $settings['showredirects'] ) ) {
-			$ret['prop'][ApiBase::PARAM_TYPE][] = 'redirect';
-			$ret['prop'][ApiBase::PARAM_DFLT] .= '|redirect';
+			$ret['prop'][ParamValidator::PARAM_TYPE][] = 'redirect';
+			$ret['prop'][ParamValidator::PARAM_DEFAULT] .= '|redirect';
 		}
 		if ( isset( $settings['props'] ) ) {
-			$ret['prop'][ApiBase::PARAM_TYPE] = array_merge(
-				$ret['prop'][ApiBase::PARAM_TYPE], $settings['props']
+			$ret['prop'][ParamValidator::PARAM_TYPE] = array_merge(
+				$ret['prop'][ParamValidator::PARAM_TYPE], $settings['props']
 			);
 		}
 
@@ -418,8 +455,8 @@ class ApiQueryBacklinksprop extends ApiQueryGeneratorBase {
 		}
 		if ( $show ) {
 			$ret['show'] = [
-				ApiBase::PARAM_TYPE => $show,
-				ApiBase::PARAM_ISMULTI => true,
+				ParamValidator::PARAM_TYPE => $show,
+				ParamValidator::PARAM_ISMULTI => true,
 			];
 		} else {
 			unset( $ret['show'] );
