@@ -20,6 +20,17 @@
  * @file
  */
 
+use MediaWiki\Content\IContentHandlerFactory;
+use MediaWiki\Content\Renderer\ContentRenderer;
+use MediaWiki\Content\Transform\ContentTransformer;
+use MediaWiki\ParamValidator\TypeDef\UserDef;
+use MediaWiki\Revision\RevisionRecord;
+use MediaWiki\Revision\RevisionStore;
+use MediaWiki\Revision\SlotRoleRegistry;
+use MediaWiki\Storage\NameTableAccessException;
+use MediaWiki\Storage\NameTableStore;
+use Wikimedia\ParamValidator\ParamValidator;
+
 /**
  * A query action to enumerate revisions of a given page, or show top revisions
  * of multiple pages. Various pieces of information may be shown - flags,
@@ -30,53 +41,53 @@
  */
 class ApiQueryRevisions extends ApiQueryRevisionsBase {
 
-	private $token = null;
+	/** @var RevisionStore */
+	private $revisionStore;
 
-	public function __construct( ApiQuery $query, $moduleName ) {
-		parent::__construct( $query, $moduleName, 'rv' );
-	}
+	/** @var NameTableStore */
+	private $changeTagDefStore;
 
-	private $tokenFunctions;
-
-	/** @deprecated since 1.24 */
-	protected function getTokenFunctions() {
-		// tokenname => function
-		// function prototype is func($pageid, $title, $rev)
-		// should return token or false
-
-		// Don't call the hooks twice
-		if ( isset( $this->tokenFunctions ) ) {
-			return $this->tokenFunctions;
-		}
-
-		// If we're in a mode that breaks the same-origin policy, no tokens can
-		// be obtained
-		if ( $this->lacksSameOriginSecurity() ) {
-			return [];
-		}
-
-		$this->tokenFunctions = [
-			'rollback' => [ self::class, 'getRollbackToken' ]
-		];
-		Hooks::run( 'APIQueryRevisionsTokens', [ &$this->tokenFunctions ] );
-
-		return $this->tokenFunctions;
-	}
+	/** @var ActorMigration */
+	private $actorMigration;
 
 	/**
-	 * @deprecated since 1.24
-	 * @param int $pageid
-	 * @param Title $title
-	 * @param Revision $rev
-	 * @return bool|string
+	 * @param ApiQuery $query
+	 * @param string $moduleName
+	 * @param RevisionStore $revisionStore
+	 * @param IContentHandlerFactory $contentHandlerFactory
+	 * @param ParserFactory $parserFactory
+	 * @param SlotRoleRegistry $slotRoleRegistry
+	 * @param NameTableStore $changeTagDefStore
+	 * @param ActorMigration $actorMigration
+	 * @param ContentRenderer $contentRenderer
+	 * @param ContentTransformer $contentTransformer
 	 */
-	public static function getRollbackToken( $pageid, $title, $rev ) {
-		global $wgUser;
-		if ( !$wgUser->isAllowed( 'rollback' ) ) {
-			return false;
-		}
-
-		return $wgUser->getEditToken( 'rollback' );
+	public function __construct(
+		ApiQuery $query,
+		$moduleName,
+		RevisionStore $revisionStore,
+		IContentHandlerFactory $contentHandlerFactory,
+		ParserFactory $parserFactory,
+		SlotRoleRegistry $slotRoleRegistry,
+		NameTableStore $changeTagDefStore,
+		ActorMigration $actorMigration,
+		ContentRenderer $contentRenderer,
+		ContentTransformer $contentTransformer
+	) {
+		parent::__construct(
+			$query,
+			$moduleName,
+			'rv',
+			$revisionStore,
+			$contentHandlerFactory,
+			$parserFactory,
+			$slotRoleRegistry,
+			$contentRenderer,
+			$contentTransformer
+		);
+		$this->revisionStore = $revisionStore;
+		$this->changeTagDefStore = $changeTagDefStore;
+		$this->actorMigration = $actorMigration;
 	}
 
 	protected function run( ApiPageSet $resultPageSet = null ) {
@@ -107,7 +118,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 
 		if ( $revCount > 0 && $enumRevMode ) {
 			$this->dieWithError(
-				[ 'apierror-revisions-nolist', $this->getModulePrefix() ], 'invalidparammix'
+				[ 'apierror-revisions-norevids', $this->getModulePrefix() ], 'invalidparammix'
 			);
 		}
 
@@ -126,20 +137,24 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 
 		$db = $this->getDB();
 
+		$idField = 'rev_id';
+		$tsField = 'rev_timestamp';
+		$pageField = 'rev_page';
+
+		$ignoreIndex = [
+			// T224017: `rev_timestamp` is never the correct index to use for this module, but
+			// MariaDB sometimes insists on trying to use it anyway. Tell it not to.
+			// Last checked with MariaDB 10.4.13
+			'revision' => 'rev_timestamp',
+		];
+		$useIndex = [];
 		if ( $resultPageSet === null ) {
 			$this->parseParameters( $params );
-			$this->token = $params['token'];
-			$opts = [];
-			if ( $this->token !== null || $pageCount > 0 ) {
-				$opts[] = 'page';
-			}
-			if ( $this->fetchContent ) {
-				$opts[] = 'text';
-			}
+			$opts = [ 'page' ];
 			if ( $this->fld_user ) {
 				$opts[] = 'user';
 			}
-			$revQuery = Revision::getQueryInfo( $opts );
+			$revQuery = $this->revisionStore->getQueryInfo( $opts );
 			$this->addTables( $revQuery['tables'] );
 			$this->addFields( $revQuery['fields'] );
 			$this->addJoinConds( $revQuery['joins'] );
@@ -148,34 +163,37 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			// Always join 'page' so orphaned revisions are filtered out
 			$this->addTables( [ 'revision', 'page' ] );
 			$this->addJoinConds(
-				[ 'page' => [ 'INNER JOIN', [ 'page_id = rev_page' ] ] ]
+				[ 'page' => [ 'JOIN', [ 'page_id = rev_page' ] ] ]
 			);
-			$this->addFields( [ 'rev_id', 'rev_timestamp', 'rev_page' ] );
+			$this->addFields( [
+				'rev_id' => $idField, 'rev_timestamp' => $tsField, 'rev_page' => $pageField
+			] );
 		}
 
 		if ( $this->fld_tags ) {
-			$this->addTables( 'tag_summary' );
-			$this->addJoinConds(
-				[ 'tag_summary' => [ 'LEFT JOIN', [ 'rev_id=ts_rev_id' ] ] ]
-			);
-			$this->addFields( 'ts_tags' );
+			$this->addFields( [ 'ts_tags' => ChangeTags::makeTagSummarySubquery( 'revision' ) ] );
 		}
 
 		if ( $params['tag'] !== null ) {
 			$this->addTables( 'change_tag' );
 			$this->addJoinConds(
-				[ 'change_tag' => [ 'INNER JOIN', [ 'rev_id=ct_rev_id' ] ] ]
+				[ 'change_tag' => [ 'JOIN', [ 'rev_id=ct_rev_id' ] ] ]
 			);
-			$this->addWhereFld( 'ct_tag', $params['tag'] );
+			try {
+				$this->addWhereFld( 'ct_tag_id', $this->changeTagDefStore->getId( $params['tag'] ) );
+			} catch ( NameTableAccessException $exception ) {
+				// Return nothing.
+				$this->addWhere( '1=0' );
+			}
 		}
 
 		if ( $resultPageSet === null && $this->fetchContent ) {
 			// For each page we will request, the user must have read rights for that page
-			$user = $this->getUser();
 			$status = Status::newGood();
+
 			/** @var Title $title */
 			foreach ( $pageSet->getGoodTitles() as $title ) {
-				if ( !$title->userCan( 'read', $user ) ) {
+				if ( !$this->getAuthority()->authorizeRead( 'read', $title ) ) {
 					$status->fatal( ApiMessage::create(
 						[ 'apierror-cannotviewtitle', wfEscapeWikiText( $title->getPrefixedText() ) ],
 						'accessdenied'
@@ -190,6 +208,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 		if ( $enumRevMode ) {
 			// Indexes targeted:
 			//  page_timestamp if we don't have rvuser
+			//  page_actor_timestamp (on revision_actor_temp) if we have rvuser in READ_NEW mode
 			//  page_user_timestamp if we have a logged-in rvuser
 			//  page_timestamp or usertext_timestamp if we have an IP rvuser
 
@@ -205,9 +224,9 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 				$continueTimestamp = $db->addQuotes( $db->timestamp( $cont[0] ) );
 				$continueId = (int)$cont[1];
 				$this->dieContinueUsageIf( $continueId != $cont[1] );
-				$this->addWhere( "rev_timestamp $op $continueTimestamp OR " .
-					"(rev_timestamp = $continueTimestamp AND " .
-					"rev_id $op= $continueId)"
+				$this->addWhere( "$tsField $op $continueTimestamp OR " .
+					"($tsField = $continueTimestamp AND " .
+					"$idField $op= $continueId)"
 				);
 			}
 
@@ -234,7 +253,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 						[ 'ar_rev_id' => $revids ],
 						__METHOD__
 					),
-				], false );
+				], $db::UNION_DISTINCT );
 				$res = $db->query( $sql, __METHOD__ );
 				foreach ( $res as $row ) {
 					if ( (int)$row->id === (int)$params['startid'] ) {
@@ -244,37 +263,43 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 						$params['end'] = $row->ts;
 					}
 				}
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 				if ( $params['startid'] !== null && $params['start'] === null ) {
 					$p = $this->encodeParamName( 'startid' );
 					$this->dieWithError( [ 'apierror-revisions-badid', $p ], "badid_$p" );
 				}
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 				if ( $params['endid'] !== null && $params['end'] === null ) {
 					$p = $this->encodeParamName( 'endid' );
 					$this->dieWithError( [ 'apierror-revisions-badid', $p ], "badid_$p" );
 				}
 
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 				if ( $params['start'] !== null ) {
 					$op = ( $params['dir'] === 'newer' ? '>' : '<' );
+					// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 					$ts = $db->addQuotes( $db->timestampOrNull( $params['start'] ) );
 					if ( $params['startid'] !== null ) {
-						$this->addWhere( "rev_timestamp $op $ts OR "
-							. "rev_timestamp = $ts AND rev_id $op= " . intval( $params['startid'] ) );
+						$this->addWhere( "$tsField $op $ts OR "
+							. "$tsField = $ts AND $idField $op= " . (int)$params['startid'] );
 					} else {
-						$this->addWhere( "rev_timestamp $op= $ts" );
+						$this->addWhere( "$tsField $op= $ts" );
 					}
 				}
+				// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 				if ( $params['end'] !== null ) {
 					$op = ( $params['dir'] === 'newer' ? '<' : '>' ); // Yes, opposite of the above
+					// @phan-suppress-next-line PhanTypePossiblyInvalidDimOffset False positive
 					$ts = $db->addQuotes( $db->timestampOrNull( $params['end'] ) );
 					if ( $params['endid'] !== null ) {
-						$this->addWhere( "rev_timestamp $op $ts OR "
-							. "rev_timestamp = $ts AND rev_id $op= " . intval( $params['endid'] ) );
+						$this->addWhere( "$tsField $op $ts OR "
+							. "$tsField = $ts AND $idField $op= " . (int)$params['endid'] );
 					} else {
-						$this->addWhere( "rev_timestamp $op= $ts" );
+						$this->addWhere( "$tsField $op= $ts" );
 					}
 				}
 			} else {
-				$this->addTimestampWhereRange( 'rev_timestamp', $params['dir'],
+				$this->addTimestampWhereRange( $tsField, $params['dir'],
 					$params['start'], $params['end'] );
 			}
 
@@ -282,28 +307,34 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			$this->addOption( 'ORDER BY', [ "rev_timestamp $sort", "rev_id $sort" ] );
 
 			// There is only one ID, use it
-			$ids = array_keys( $pageSet->getGoodTitles() );
-			$this->addWhereFld( 'rev_page', reset( $ids ) );
+			$ids = array_keys( $pageSet->getGoodPages() );
+			$this->addWhereFld( $pageField, reset( $ids ) );
 
 			if ( $params['user'] !== null ) {
-				$actorQuery = ActorMigration::newMigration()
-					->getWhere( $db, 'rev_user', User::newFromName( $params['user'], false ) );
+				$actorQuery = $this->actorMigration->getWhere( $db, 'rev_user', $params['user'] );
 				$this->addTables( $actorQuery['tables'] );
 				$this->addJoinConds( $actorQuery['joins'] );
 				$this->addWhere( $actorQuery['conds'] );
 			} elseif ( $params['excludeuser'] !== null ) {
-				$actorQuery = ActorMigration::newMigration()
-					->getWhere( $db, 'rev_user', User::newFromName( $params['excludeuser'], false ) );
+				$actorQuery = $this->actorMigration->getWhere( $db, 'rev_user', $params['excludeuser'] );
 				$this->addTables( $actorQuery['tables'] );
 				$this->addJoinConds( $actorQuery['joins'] );
 				$this->addWhere( 'NOT(' . $actorQuery['conds'] . ')' );
+			} else {
+				// T258480: MariaDB ends up using rev_page_actor_timestamp in some cases here.
+				// Last checked with MariaDB 10.4.13
+				// Unless we are filtering by user (see above), we always want to use the
+				// "history" index on the revision table, namely page_timestamp.
+				$useIndex['revision'] = 'rev_page_timestamp';
 			}
+
 			if ( $params['user'] !== null || $params['excludeuser'] !== null ) {
 				// Paranoia: avoid brute force searches (T19342)
-				if ( !$this->getUser()->isAllowed( 'deletedhistory' ) ) {
-					$bitmask = Revision::DELETED_USER;
-				} elseif ( !$this->getUser()->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
-					$bitmask = Revision::DELETED_USER | Revision::DELETED_RESTRICTED;
+				if ( !$this->getAuthority()->isAllowed( 'deletedhistory' ) ) {
+					$bitmask = RevisionRecord::DELETED_USER;
+				} elseif ( !$this->getAuthority()->isAllowedAny( 'suppressrevision', 'viewsuppressed' )
+				) {
+					$bitmask = RevisionRecord::DELETED_USER | RevisionRecord::DELETED_RESTRICTED;
 				} else {
 					$bitmask = 0;
 				}
@@ -320,28 +351,28 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			$this->addWhereFld( 'rev_id', array_keys( $revs ) );
 
 			if ( $params['continue'] !== null ) {
-				$this->addWhere( 'rev_id >= ' . intval( $params['continue'] ) );
+				$this->addWhere( 'rev_id >= ' . (int)$params['continue'] );
 			}
 			$this->addOption( 'ORDER BY', 'rev_id' );
 		} elseif ( $pageCount > 0 ) {
 			// Always targets the rev_page_id index
 
-			$titles = $pageSet->getGoodTitles();
+			$pageids = array_keys( $pageSet->getGoodPages() );
 
 			// When working in multi-page non-enumeration mode,
 			// limit to the latest revision only
 			$this->addWhere( 'page_latest=rev_id' );
 
 			// Get all page IDs
-			$this->addWhereFld( 'page_id', array_keys( $titles ) );
+			$this->addWhereFld( 'page_id', $pageids );
 			// Every time someone relies on equality propagation, god kills a kitten :)
-			$this->addWhereFld( 'rev_page', array_keys( $titles ) );
+			$this->addWhereFld( 'rev_page', $pageids );
 
 			if ( $params['continue'] !== null ) {
 				$cont = explode( '|', $params['continue'] );
 				$this->dieContinueUsageIf( count( $cont ) != 2 );
-				$pageid = intval( $cont[0] );
-				$revid = intval( $cont[1] );
+				$pageid = (int)$cont[0];
+				$revid = (int)$cont[1];
 				$this->addWhere(
 					"rev_page > $pageid OR " .
 					"(rev_page = $pageid AND " .
@@ -358,6 +389,12 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 
 		$this->addOption( 'LIMIT', $this->limit + 1 );
 
+		$this->addOption( 'IGNORE INDEX', $ignoreIndex );
+
+		if ( $useIndex ) {
+			$this->addOption( 'USE INDEX', $useIndex );
+		}
+
 		$count = 0;
 		$generated = [];
 		$hookData = [];
@@ -369,12 +406,12 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 				// additional pages to be had. Stop here...
 				if ( $enumRevMode ) {
 					$this->setContinueEnumParameter( 'continue',
-						$row->rev_timestamp . '|' . intval( $row->rev_id ) );
+						$row->rev_timestamp . '|' . (int)$row->rev_id );
 				} elseif ( $revCount > 0 ) {
-					$this->setContinueEnumParameter( 'continue', intval( $row->rev_id ) );
+					$this->setContinueEnumParameter( 'continue', (int)$row->rev_id );
 				} else {
-					$this->setContinueEnumParameter( 'continue', intval( $row->rev_page ) .
-						'|' . intval( $row->rev_id ) );
+					$this->setContinueEnumParameter( 'continue', (int)$row->rev_page .
+						'|' . (int)$row->rev_id );
 				}
 				break;
 			}
@@ -382,33 +419,19 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			if ( $resultPageSet !== null ) {
 				$generated[] = $row->rev_id;
 			} else {
-				$revision = new Revision( $row );
+				$revision = $this->revisionStore->newRevisionFromRow( $row, 0, Title::newFromRow( $row ) );
 				$rev = $this->extractRevisionInfo( $revision, $row );
-
-				if ( $this->token !== null ) {
-					$title = $revision->getTitle();
-					$tokenFunctions = $this->getTokenFunctions();
-					foreach ( $this->token as $t ) {
-						$val = call_user_func( $tokenFunctions[$t], $title->getArticleID(), $title, $revision );
-						if ( $val === false ) {
-							$this->addWarning( [ 'apiwarn-tokennotallowed', $t ] );
-						} else {
-							$rev[$t . 'token'] = $val;
-						}
-					}
-				}
-
 				$fit = $this->processRow( $row, $rev, $hookData ) &&
 					$this->addPageSubItem( $row->rev_page, $rev, 'rev' );
 				if ( !$fit ) {
 					if ( $enumRevMode ) {
 						$this->setContinueEnumParameter( 'continue',
-							$row->rev_timestamp . '|' . intval( $row->rev_id ) );
+							$row->rev_timestamp . '|' . (int)$row->rev_id );
 					} elseif ( $revCount > 0 ) {
-						$this->setContinueEnumParameter( 'continue', intval( $row->rev_id ) );
+						$this->setContinueEnumParameter( 'continue', (int)$row->rev_id );
 					} else {
-						$this->setContinueEnumParameter( 'continue', intval( $row->rev_page ) .
-							'|' . intval( $row->rev_id ) );
+						$this->setContinueEnumParameter( 'continue', (int)$row->rev_page .
+							'|' . (int)$row->rev_id );
 					}
 					break;
 				}
@@ -420,34 +443,27 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 		}
 	}
 
-	public function getCacheMode( $params ) {
-		if ( isset( $params['token'] ) ) {
-			return 'private';
-		}
-		return parent::getCacheMode( $params );
-	}
-
 	public function getAllowedParams() {
 		$ret = parent::getAllowedParams() + [
 			'startid' => [
-				ApiBase::PARAM_TYPE => 'integer',
+				ParamValidator::PARAM_TYPE => 'integer',
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'endid' => [
-				ApiBase::PARAM_TYPE => 'integer',
+				ParamValidator::PARAM_TYPE => 'integer',
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'start' => [
-				ApiBase::PARAM_TYPE => 'timestamp',
+				ParamValidator::PARAM_TYPE => 'timestamp',
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'end' => [
-				ApiBase::PARAM_TYPE => 'timestamp',
+				ParamValidator::PARAM_TYPE => 'timestamp',
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'dir' => [
-				ApiBase::PARAM_DFLT => 'older',
-				ApiBase::PARAM_TYPE => [
+				ParamValidator::PARAM_DEFAULT => 'older',
+				ParamValidator::PARAM_TYPE => [
 					'newer',
 					'older'
 				],
@@ -455,19 +471,18 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'user' => [
-				ApiBase::PARAM_TYPE => 'user',
+				ParamValidator::PARAM_TYPE => 'user',
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_RETURN_OBJECT => true,
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'excludeuser' => [
-				ApiBase::PARAM_TYPE => 'user',
+				ParamValidator::PARAM_TYPE => 'user',
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_RETURN_OBJECT => true,
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'tag' => null,
-			'token' => [
-				ApiBase::PARAM_DEPRECATED => true,
-				ApiBase::PARAM_TYPE => array_keys( $this->getTokenFunctions() ),
-				ApiBase::PARAM_ISMULTI => true
-			],
 			'continue' => [
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-continue',
 			],
@@ -481,7 +496,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 	protected function getExamplesMessages() {
 		return [
 			'action=query&prop=revisions&titles=API|Main%20Page&' .
-				'rvprop=timestamp|user|comment|content'
+				'rvslots=*&rvprop=timestamp|user|comment|content'
 				=> 'apihelp-query+revisions-example-content',
 			'action=query&prop=revisions&titles=Main%20Page&rvlimit=5&' .
 				'rvprop=timestamp|user|comment'
