@@ -44,7 +44,7 @@ use Wikimedia\NormalizedException\NormalizedException;
  * @ingroup SpecialPage
  */
 class WikiImporter {
-	/** @var XMLReader */
+	/** @var XMLReader|null */
 	private $reader;
 
 	/** @var array|null */
@@ -142,6 +142,7 @@ class WikiImporter {
 	 * @param IContentHandlerFactory $contentHandlerFactory
 	 * @param SlotRoleRegistry $slotRoleRegistry
 	 * @throws MWException
+	 * @suppress PhanStaticCallToNonStatic, UnusedSuppression -- for PHP 7.4 support
 	 */
 	public function __construct(
 		ImportSource $source,
@@ -156,7 +157,6 @@ class WikiImporter {
 		IContentHandlerFactory $contentHandlerFactory,
 		SlotRoleRegistry $slotRoleRegistry
 	) {
-		$this->reader = new XMLReader();
 		$this->config = $config;
 		$this->hookRunner = new HookRunner( $hookContainer );
 		$this->contentLanguage = $contentLanguage;
@@ -177,10 +177,21 @@ class WikiImporter {
 		// XMLReader::open (T86036)
 		// phpcs:ignore Generic.PHP.NoSilencedErrors -- suppress deprecation per T268847
 		$oldDisable = @libxml_disable_entity_loader( false );
-		if ( defined( 'LIBXML_PARSEHUGE' ) ) {
-			$status = $this->reader->open( "uploadsource://$id", null, LIBXML_PARSEHUGE );
+		if ( PHP_VERSION_ID >= 80000 ) {
+			// A static call is now preferred, and avoids https://github.com/php/php-src/issues/11548
+			$reader = XMLReader::open(
+				"uploadsource://$id", null, LIBXML_PARSEHUGE );
+			if ( $reader instanceof XMLReader ) {
+				$this->reader = $reader;
+				$status = true;
+			} else {
+				$status = false;
+			}
 		} else {
-			$status = $this->reader->open( "uploadsource://$id" );
+			// A static call generated a deprecation warning prior to PHP 8.0
+			$this->reader = new XMLReader;
+			$status = $this->reader->open(
+				"uploadsource://$id", null, LIBXML_PARSEHUGE );
 		}
 		if ( !$status ) {
 			$error = libxml_get_last_error();
