@@ -1,10 +1,10 @@
 <?php
-/** Chinese (Taiwan) (‪中文(台灣)‬)
+/** Chinese (Taiwan) (‪中文(臺灣)‬)
  *
  * To improve a translation please visit https://translatewiki.net
  *
- * @ingroup Language
  * @file
+ * @ingroup Languages
  *
  * @author Alexsh
  * @author Andrew971218
@@ -17,12 +17,13 @@
  * @author Roc michael
  * @author Shizhao
  * @author Urhixidur
+ * @author Winston Sung
  * @author Wong128hk
  * @author Zerng07
  * @author לערי ריינהארט
  */
 
-$fallback = 'zh-hant, zh-hans';
+$fallback = 'zh-hant, zh-hk, zh, zh-hans';
 
 $namespaceNames = [
 	NS_USER             => '使用者',
@@ -38,6 +39,7 @@ $namespaceAliases = [
 	"圖片討論" => NS_FILE_TALK,
 ];
 
+/** @phpcs-require-sorted-array */
 $specialPageAliases = [
 	'Allmessages'               => [ '所有訊息' ],
 	'Ancientpages'              => [ '最舊頁面' ],
