@@ -24,6 +24,9 @@
  * @ingroup Maintenance
  */
 
+use MediaWiki\Linker\LinkTarget;
+use MediaWiki\MediaWikiServices;
+
 require_once __DIR__ . '/Maintenance.php';
 
 /**
@@ -32,16 +35,34 @@ require_once __DIR__ . '/Maintenance.php';
  * @ingroup Maintenance
  */
 class BackupReader extends Maintenance {
+	/** @var int */
 	public $reportingInterval = 100;
+	/** @var int */
 	public $pageCount = 0;
+	/** @var int */
 	public $revCount = 0;
+	/** @var bool */
 	public $dryRun = false;
+	/** @var bool */
 	public $uploads = false;
+	/** @var int */
 	protected $uploadCount = 0;
+	/** @var string|false */
 	public $imageBasePath = false;
+	/** @var array|false */
 	public $nsFilter = false;
+	/** @var resource|false */
+	public $stderr;
+	/** @var callable|null */
+	protected $importCallback;
+	/** @var callable|null */
+	protected $logItemCallback;
+	/** @var callable|null */
+	protected $uploadCallback;
+	/** @var float */
+	protected $startTime;
 
-	function __construct() {
+	public function __construct() {
 		parent::__construct();
 		$gz = in_array( 'compress.zlib', stream_get_wrappers() )
 			? 'ok'
@@ -82,7 +103,7 @@ TEXT
 		);
 		$this->addOption( 'image-base-path', 'Import files from a specified path', false, true );
 		$this->addOption( 'skip-to', 'Start from nth page by skipping first n-1 pages', false, true );
-		$this->addOption( 'username-interwiki', 'Use interwiki usernames with this prefix', false, true );
+		$this->addOption( 'username-prefix', 'Prefix for interwiki usernames', false, true );
 		$this->addOption( 'no-local-users',
 			'Treat all usernames as interwiki. ' .
 			'The default is to assign edits to local users where they exist.',
@@ -92,17 +113,19 @@ TEXT
 	}
 
 	public function execute() {
-		if ( wfReadOnly() ) {
+		if ( MediaWikiServices::getInstance()->getReadOnlyMode()->isReadOnly() ) {
 			$this->fatalError( "Wiki is in read-only mode; you'll need to disable it for import to work." );
 		}
 
 		$this->reportingInterval = intval( $this->getOption( 'report', 100 ) );
 		if ( !$this->reportingInterval ) {
-			$this->reportingInterval = 100; // avoid division by zero
+			// avoid division by zero
+			$this->reportingInterval = 100;
 		}
 
 		$this->dryRun = $this->hasOption( 'dry-run' );
-		$this->uploads = $this->hasOption( 'uploads' ); // experimental!
+		$this->uploads = $this->hasOption( 'uploads' );
+
 		if ( $this->hasOption( 'image-base-path' ) ) {
 			$this->imageBasePath = $this->getOption( 'image-base-path' );
 		}
@@ -110,8 +133,8 @@ TEXT
 			$this->setNsfilter( explode( '|', $this->getOption( 'namespaces' ) ) );
 		}
 
-		if ( $this->hasArg() ) {
-			$this->importFromFile( $this->getArg() );
+		if ( $this->hasArg( 0 ) ) {
+			$this->importFromFile( $this->getArg( 0 ) );
 		} else {
 			$this->importFromStdin();
 		}
@@ -121,7 +144,7 @@ TEXT
 		$this->output( "and initSiteStats.php to update page and revision counts\n" );
 	}
 
-	function setNsfilter( array $namespaces ) {
+	private function setNsfilter( array $namespaces ) {
 		if ( count( $namespaces ) == 0 ) {
 			$this->nsFilter = false;
 
@@ -131,36 +154,25 @@ TEXT
 	}
 
 	private function getNsIndex( $namespace ) {
-		global $wgContLang;
-		$result = $wgContLang->getNsIndex( $namespace );
+		$contLang = MediaWikiServices::getInstance()->getContentLanguage();
+		$result = $contLang->getNsIndex( $namespace );
 		if ( $result !== false ) {
 			return $result;
 		}
 		$ns = intval( $namespace );
-		if ( strval( $ns ) === $namespace && $wgContLang->getNsText( $ns ) !== false ) {
+		if ( strval( $ns ) === $namespace && $contLang->getNsText( $ns ) !== false ) {
 			return $ns;
 		}
 		$this->fatalError( "Unknown namespace text / index specified: $namespace" );
 	}
 
 	/**
-	 * @param Title|Revision $obj
+	 * @param LinkTarget|null $title
 	 * @throws MWException
 	 * @return bool
 	 */
-	private function skippedNamespace( $obj ) {
-		$title = null;
-		if ( $obj instanceof Title ) {
-			$title = $obj;
-		} elseif ( $obj instanceof Revision ) {
-			$title = $obj->getTitle();
-		} elseif ( $obj instanceof WikiRevision ) {
-			$title = $obj->title;
-		} else {
-			throw new MWException( "Cannot get namespace of object in " . __METHOD__ );
-		}
-
-		if ( is_null( $title ) ) {
+	private function skippedNamespace( $title ) {
+		if ( $title === null ) {
 			// Probably a log entry
 			return false;
 		}
@@ -170,14 +182,14 @@ TEXT
 		return is_array( $this->nsFilter ) && !in_array( $ns, $this->nsFilter );
 	}
 
-	function reportPage( $page ) {
+	public function reportPage( $page ) {
 		$this->pageCount++;
 	}
 
 	/**
-	 * @param Revision $rev
+	 * @param WikiRevision $rev
 	 */
-	function handleRevision( $rev ) {
+	public function handleRevision( WikiRevision $rev ) {
 		$title = $rev->getTitle();
 		if ( !$title ) {
 			$this->progress( "Got bogus revision with null title!" );
@@ -198,12 +210,12 @@ TEXT
 	}
 
 	/**
-	 * @param Revision $revision
+	 * @param WikiRevision $revision
 	 * @return bool
 	 */
-	function handleUpload( $revision ) {
+	public function handleUpload( WikiRevision $revision ) {
 		if ( $this->uploads ) {
-			if ( $this->skippedNamespace( $revision ) ) {
+			if ( $this->skippedNamespace( $revision->getTitle() ) ) {
 				return false;
 			}
 			$this->uploadCount++;
@@ -213,7 +225,7 @@ TEXT
 			if ( !$this->dryRun ) {
 				// bluuuh hack
 				// call_user_func( $this->uploadCallback, $revision );
-				$dbw = $this->getDB( DB_MASTER );
+				$dbw = $this->getDB( DB_PRIMARY );
 
 				return $dbw->deadlockLoop( [ $revision, 'importUpload' ] );
 			}
@@ -222,8 +234,11 @@ TEXT
 		return false;
 	}
 
-	function handleLogItem( $rev ) {
-		if ( $this->skippedNamespace( $rev ) ) {
+	/**
+	 * @param WikiRevision $rev
+	 */
+	public function handleLogItem( WikiRevision $rev ) {
+		if ( $this->skippedNamespace( $rev->getTitle() ) ) {
 			return;
 		}
 		$this->revCount++;
@@ -234,13 +249,13 @@ TEXT
 		}
 	}
 
-	function report( $final = false ) {
+	private function report( $final = false ) {
 		if ( $final xor ( $this->pageCount % $this->reportingInterval == 0 ) ) {
 			$this->showReport();
 		}
 	}
 
-	function showReport() {
+	private function showReport() {
 		if ( !$this->mQuiet ) {
 			$delta = microtime( true ) - $this->startTime;
 			if ( $delta ) {
@@ -257,14 +272,14 @@ TEXT
 				$this->progress( "$this->revCount ($revrate revs/sec)" );
 			}
 		}
-		wfWaitForSlaves();
+		MediaWikiServices::getInstance()->getDBLoadBalancerFactory()->waitForReplication();
 	}
 
-	function progress( $string ) {
+	private function progress( $string ) {
 		fwrite( $this->stderr, $string . "\n" );
 	}
 
-	function importFromFile( $filename ) {
+	private function importFromFile( $filename ) {
 		if ( preg_match( '/\.gz$/', $filename ) ) {
 			$filename = 'compress.zlib://' . $filename;
 		} elseif ( preg_match( '/\.bz2$/', $filename ) ) {
@@ -274,11 +289,14 @@ TEXT
 		}
 
 		$file = fopen( $filename, 'rt' );
+		if ( $file === false ) {
+			$this->fatalError( error_get_last()['message'] ?? 'Could not open file' );
+		}
 
 		return $this->importFromHandle( $file );
 	}
 
-	function importFromStdin() {
+	private function importFromStdin() {
 		$file = fopen( 'php://stdin', 'rt' );
 		if ( self::posix_isatty( $file ) ) {
 			$this->maybeHelp( true );
@@ -287,11 +305,13 @@ TEXT
 		return $this->importFromHandle( $file );
 	}
 
-	function importFromHandle( $handle ) {
+	private function importFromHandle( $handle ) {
 		$this->startTime = microtime( true );
 
 		$source = new ImportStreamSource( $handle );
-		$importer = new WikiImporter( $source, $this->getConfig() );
+		$importer = MediaWikiServices::getInstance()
+			->getWikiImporterFactory()
+			->getWikiImporter( $source );
 
 		// Updating statistics require a lot of time so disable it
 		$importer->disableStatisticsUpdate();
@@ -312,8 +332,7 @@ TEXT
 			$statusRootPage = $importer->setTargetRootPage( $this->getOption( 'rootpage' ) );
 			if ( !$statusRootPage->isGood() ) {
 				// Die here so that it doesn't print "Done!"
-				$this->fatalError( $statusRootPage->getMessage()->text() );
-				return false;
+				$this->fatalError( $statusRootPage->getMessage( false, false, 'en' )->text() );
 			}
 		}
 		if ( $this->hasOption( 'skip-to' ) ) {
@@ -322,7 +341,7 @@ TEXT
 			$this->pageCount = $nthPage - 1;
 		}
 		$importer->setPageCallback( [ $this, 'reportPage' ] );
-		$importer->setNoticeCallback( function ( $msg, $params ) {
+		$importer->setNoticeCallback( static function ( $msg, $params ) {
 			echo wfMessage( $msg, $params )->text() . "\n";
 		} );
 		$this->importCallback = $importer->setRevisionCallback(
