@@ -60,8 +60,6 @@ class MWDocGen extends Maintenance {
 	private $excludePatterns;
 	/** @var bool */
 	private $doDot;
-	/** @var bool */
-	private $doMan;
 
 	/**
 	 * Prepare Maintenance class
@@ -102,7 +100,7 @@ class MWDocGen extends Maintenance {
 		$this->input = '';
 		$inputs = explode( ',', $this->getOption( 'file', '' ) );
 		foreach ( $inputs as $input ) {
-			# Doxygen inputs are space separted and double quoted
+			# Doxygen inputs are space separated and double quoted
 			$this->input .= " \"$IP/$input\"";
 		}
 
@@ -136,7 +134,7 @@ class MWDocGen extends Maintenance {
 			}
 		}
 
-		$this->doDot = shell_exec( 'which dot' );
+		$this->doDot = (bool)shell_exec( 'which dot' );
 	}
 
 	public function execute() {
